@@
@@ Copyright (c) 2005-2006 Jannis Harder
@@ 
@@ Permission is  hereby granted, free  of charge,  to any person  obtaining a
@@ copy of this software and  associated documentation files (the "Software"),
@@ to deal in  the Software without restriction,  including without limitation
@@ the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
@@ and/or  sell copies  of the  Software, and  to permit  persons to  whom the
@@ Software is furnished to do so subject to the following conditions:
@@ 
@@ The above copyright notice and this  permission notice shall be included in
@@ all copies or substantial portions of the Software.
@@ 
@@ THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
@@ IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
@@ FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ AUTHORS OR  COPYRIGHT HOLDERS  BE LIABLE  FOR ANY  CLAIM, DAMAGES  OR OTHER
@@ LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
@@ FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
@@ DEALINGS IN THE SOFTWARE.
@@


.align 4
.arm

.equ fracp, 29 @ 29 fractional bits
.equ intp, 2 @ 2 integer bits
.equ sintp, (intp+1) @ 2 integer bits + 1 sign

.equ width, 320
.equ height, 240

.global render_mandelbrot
render_mandelbrot:
  mov ip, sp
  stmfd sp!, {r4, r5, r6, r7, r8, r9, r10, fp, ip, lr, pc } @ save registers
  sub fp, ip, #4
  @ register usage
  @ r0 xmin
  @ r1 y (creal)
  @ r2 x-delta
  @ r3 y-delta
  @ r4 x-int
  @ r5 y-int
  @ r6 x (cimag)
  @ r7 zreal
  @ r8 zimag
  @ r9 iteration count
  @ r10 tmp0
  @ ip tmp1
  @ sp tmp2 
  
  
  mov r5, #0
  
line:
  mov r6, r0 @ (re)init cimag
  mov r4, #0 @ (re)init x-int
pixel:
  mov r7, #0 @ init z = 0
  mov r8, #0
  
  @mov r9, #7 @ we want 10 iterations
  
  ldr r9, [fp, #4] @ load iteration count
  
iteration:
  
  @ z = z * z + c
  
  @ zr = zr * zr - zi * zi + cr
  mov r10, r8 @ make a copy of zr
  
  smull sp, r8, r10, r10 @ square zr
  @mov r8, r8, lsr #fracp
  @orr r8, r8, sp, lsl #sintp

  
  
  smull ip, sp, r7, r7 @ square zi
  @sub r8, r8, ip, lsr #fracp @ and sub it
  sub r8, r8, sp
  
  add r8, r6, r8, lsl #sintp @ add cr
  
  @ zi = 2 * zi * zr + ci
  
  smull sp, r7, r10, r7 @ zr*zi*2
  @mov r7, r7, lsr #(fracp-1)
  @orr r7, r7, sp, lsl #(sintp+1)
  
  add r7, r1, r7, lsl #(sintp+1)
  
  @ |z| > 2 test
  smull ip, sp, r7,r7 @square real


  smull r10, ip, r8,r8 @ add square imag
  add ip, ip, sp

  mov sp, #((2*2) << (fracp*2-32)) @ >= 2**2?
  cmp ip, sp

  mov sp, #0

  sublt r9, r9, #1
  cmplt sp, r9
  blt iteration
  
  
end_iterations:
  ldr ip, [fp, #8] @load screen buffer address
  add ip, ip, r5
  add ip, ip, r4
  
  strb r9, [ip] @ write iteration count
  
  @ next pixel
  add r4, r4, #1
  
  mov sp, #width @ next line?
  cmp r4, sp 
  
  @no new line
  addnes r6, r6, r2 @ add delta-x to x
  bne pixel @ next pixel
  
  @a new line
  
  add r5, r5, #width
  
  mov sp, #width*height @ done?
  cmp r5, sp
  
  @no
  addne r1, r1, r3 @ add delta-y to y
  bne line @ next line
  
  @we're done!
  
  ldmea fp, {r4, r5, r6, r7, r8, r9, r10, fp, sp, pc } @ restore and return

.pool


