#include "minimal.h"

/*
 * Copyright (c) 2005-2006 Jannis Harder
 * 
 * Permission is  hereby granted, free  of charge,  to any person  obtaining a
 * copy of this software and  associated documentation files (the "Software"),
 * to deal in  the Software without restriction,  including without limitation
 * the rights  to use, copy,  modify, merge, publish,  distribute, sublicense,
 * and/or  sell copies  of the  Software, and  to permit  persons to  whom the
 * Software is furnished to do so subject to the following conditions:
 * 
 * The above copyright notice and this  permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS",  WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE WARRANTIES  OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR  COPYRIGHT HOLDERS  BE LIABLE  FOR ANY  CLAIM, DAMAGES  OR OTHER
 * LIABILITY, WHETHER  IN AN  ACTION OF CONTRACT,  TORT OR  OTHERWISE, ARISING
 * FROM,  OUT OF  OR IN  CONNECTION  WITH THE  SOFTWARE  OR THE  USE OR  OTHER
 * DEALINGS IN THE SOFTWARE.
 */


#define XY(X,Y) (320*Y+X)

#define FIXED(Y) (int)((Y)*(1<<29))


#define GP2X_ANY (GP2X_PUSH | GP2X_A | GP2X_B | GP2X_X | GP2X_Y | GP2X_R | GP2X_L | GP2X_START | GP2X_SELECT)

void render_mandelbrot(int xmin, int ymin, int xdelta, int ydelta, int iterations, unsigned char *gp2x_screen8);


int test(int a,int b, int c, int e) {
  return e;
}

unsigned char curs[] = {
  1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  1,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
  1,2,2,1,0,0,0,0,0,0,0,0,0,0,0,0,
  1,2,2,2,1,0,0,0,0,0,0,0,0,0,0,0,
  1,2,2,2,2,1,0,0,0,0,0,0,0,0,0,0,
  1,2,2,2,2,2,1,0,0,0,0,0,0,0,0,0,
  1,2,2,2,2,2,2,1,0,0,0,0,0,0,0,0,
  1,2,2,2,2,2,2,2,1,0,0,0,0,0,0,0,
  1,2,2,2,1,1,1,1,1,1,0,0,0,0,0,0,
  1,2,2,1,0,0,0,0,0,0,0,0,0,0,0,0,
  1,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
  1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};

const unsigned char wave[] = {0x00,0x22,0x44,0x66,0x88,0xAA,0xCC,0xEE,0xFF,0xDD,0xBB,0x99,0x77,0x55,0x33,0x11};


int main(int argc, char *argv[]) {
  gp2x_init(60, 8, 11025,16,1,60, 1);
  
  gp2x_video_RGB_color8(0,0,0,0); // black
  gp2x_video_RGB_color8(255,255,255,255); // black
  gp2x_video_RGB_setpalette();
  
  gp2x_printf(NULL,5,0 ,"Mandelbrot v0.2 by Jannis (jix) Harder");
  gp2x_printf(NULL,5,15,"Features:");
  gp2x_printf(NULL,5,25,"\xB7 signed 2.29..2.26 fixed point math.");
  gp2x_printf(NULL,5,35,"\xB7 hand written assembler engine.");
  gp2x_printf(NULL,5,45,"\xB7 6 different palettes.");
  gp2x_printf(NULL,5,60,"Using rlyeh's Minimal Library v0.B (preview)");
  gp2x_printf(NULL,109,115,"\xAB PRESS ANY KEY \xBB");
  gp2x_printf(NULL,5,155,"R/L: palette # -/+: iteration count");
  gp2x_printf(NULL,5,165,"B/X: zoom in/out # A: move to # Y: reset");
  gp2x_printf(NULL,5,175,"START: exit # JOYSTICK: move cursor");
  gp2x_printf(NULL,5,185,"SELECT+ANY KEY: fast ANY KEY");
  gp2x_printf(NULL,5,200,"e-mail: jannis@harderweb.de");
  gp2x_printf(NULL,5,215,"Copyright \xA9 2005-2006 Jannis Harder.");
  gp2x_printf(NULL,5,225,"See README.txt");
  
  gp2x_video_logo_enable(1);
  
  gp2x_video_RGB_flip(0);
  
  
  while(!(gp2x_joystick_read()&GP2X_ANY))
    gp2x_timer_delay(1);
  while(gp2x_joystick_read()&GP2X_ANY)
    gp2x_timer_delay(1);
  
  gp2x_video_logo_enable(0);
  gp2x_video_cursor_setup(curs,16,1,255,255,255,0xF,2,0,0,0,0xF);
  
  
  float center_x = -0.75;
  float center_y = 0.0;
  

  float rad_y = 1.6;
  
  int show_cursor = 0;
  
  int palette_num = 0;
  
  int rerender = 1;
  int repalette = 1;
  
  int iteration_count = 50;

  int ccursor = 0;

  int cx = 160;
  int cy = 120;
  
  int nomove = 0;
  
  unsigned long last_pad = 0;

  

  

  
  
  while(1) {
    
    int pc;
    
    
    if(ccursor) {
      ccursor = 0;
      gp2x_video_cursor_show(show_cursor);
    }
      
    
    
    if(repalette) {
      
      switch(palette_num) {

      case 0:
        gp2x_video_RGB_color8(0,0,0,0); // black
        
        for(pc = 1; pc < 256; pc++) 
          gp2x_video_RGB_color8((pc-iteration_count-1)&0xFF,((~(pc-iteration_count-1))<<4)&0xFF,0,pc);

      break;
      case 1:
        gp2x_video_RGB_color8(0,0,255,0); // blue

        for(pc = 1; pc < 256; pc++) 
          gp2x_video_RGB_color8((pc-iteration_count-1)&0xFF,((~(pc-iteration_count-1))<<4)&0xFF,255,pc);
      break;
      case 2:
        gp2x_video_RGB_color8(127,127,127,0); // gray
        
        for(pc = 1; pc < 256; pc++) {
          int z = ((pc^iteration_count)&1 ? 255 : 0);
          gp2x_video_RGB_color8(z,z,z,pc);
        }
      break;
      case 3:
        gp2x_video_RGB_color8(0,0,255,0); // gray

        for(pc = 1; pc < 256; pc++) {
          int z = ((pc^iteration_count)&1 ? 255 : 0);
          gp2x_video_RGB_color8(0,z,((pc-iteration_count-1)<<2)&0xFF,pc);
        }
      break;
      case 4:
        gp2x_video_RGB_color8(0,0,0,0); // black

        for(pc = 1; pc < 256; pc++) {
          int q = wave[(pc-iteration_count-1)&0xF];
          gp2x_video_RGB_color8(q,q,255-q,pc);
        }
      break;
      case 5:
        gp2x_video_RGB_color8(0,0,0,0); // black
        for(pc = 1; pc < 256; pc++) {
          int q = wave[(pc-iteration_count-1)&0xF];
          gp2x_video_RGB_color8(q,0,((255-q)*((pc-iteration_count-1)&0xFF))>>8,pc);
        }
      break;
      #define NUM_PALETTES 6
      }
      
    }
    

    if(rerender) {
      rerender = 0;
      float rad_x = (rad_y * 4.0) / 3.0;
      float step2 = rad_y/120;
      int step3 = FIXED(step2);
      if(!nomove) {
        center_x += step2 * (cx-160);
        center_y += step2 * (cy-120);
      }
      
      cx = 160;
      cy = 120;
      
      nomove = 0;
      render_mandelbrot(FIXED(center_x-rad_x),FIXED(center_y-rad_y),step3,step3,iteration_count,gp2x_video_RGB[0].screen8);
      
      
      gp2x_video_RGB_flip(0);
    }
    if(repalette) {
      repalette = 0;
      gp2x_video_RGB_setpalette();
    }
    
#define BUTTON_UP(X) ((last_pad & X) && (! (pad & X)))
    
    
    unsigned long pad = gp2x_joystick_read();
    
    int step = 1;
    float scale = 1.5;
    
    if(pad & GP2X_SELECT) {
      step = 10;
      scale = 3.5;

    }

    if((pad & GP2X_L && pad & GP2X_R) || pad & GP2X_START) exit(0);

    
    
    if(BUTTON_UP(GP2X_L)) {
      palette_num = (palette_num-step+NUM_PALETTES) % NUM_PALETTES;
      repalette = 1;
      show_cursor = 0;
      ccursor = 1;
    }
    
    if(BUTTON_UP(GP2X_R)) {
      palette_num = (palette_num+step) % NUM_PALETTES;
      repalette = 1;
      show_cursor = 0;
      ccursor = 1;
    }
    
    if(BUTTON_UP(GP2X_VOL_UP)) {
      if(iteration_count < 500) {
        iteration_count+=step;
        rerender = 1;
        repalette = 1;
        nomove = 1;
      }
      show_cursor = 0;
      ccursor = 1;
    }
    
    if(BUTTON_UP(GP2X_VOL_DOWN)) {
      if(iteration_count >= step) {
        iteration_count-=step;
        rerender = 1;
        repalette = 1;
        nomove = 1;
      }
      show_cursor = 0;
      ccursor = 1;
    }
    
    if(BUTTON_UP(GP2X_B)) {
      float step2 = rad_y/120;
      
      center_x += step2 * (cx-160);
      center_y += step2 * (cy-120);
      nomove = 1;
      
      rad_y /= scale;
      rerender = 1;
      show_cursor = 0;
      ccursor = 1;

    }
    
    if(BUTTON_UP(GP2X_X)) {
      
      float step2 = rad_y/120;

      center_x += step2 * (cx-160);
      center_y += step2 * (cy-120);
      nomove = 1;
      
      rad_y *= scale;
      if(rad_y > 1.6)
        rad_y = 1.6;
      rerender = 1;
      show_cursor = 0;
      ccursor = 1;

    }
    
    if(BUTTON_UP(GP2X_A)) {
      rerender = 1;
      show_cursor = 0;
      ccursor = 1;

    }
    
    if(BUTTON_UP(GP2X_Y)) {
      center_x = -0.75;
      center_y = 0.0;
      rad_y = 1.6;
      show_cursor = 0;
      palette_num = 0;
      rerender = 1;
      repalette = 1;
      iteration_count = 100;

      show_cursor = 0;

      ccursor = 1;
    }
    
    if(pad & (GP2X_UP | GP2X_DOWN | GP2X_LEFT | GP2X_RIGHT)) {
      show_cursor = 1;
      ccursor = 1;
      if(pad & GP2X_UP) 
        cy -= step;
      
      if(pad & GP2X_DOWN) 
        cy += step;
      
      if(pad & GP2X_LEFT) 
        cx -= step;
      
      if(pad & GP2X_RIGHT) 
        cx += step;
      
      if(cy<0)
        cy = 0;
      if(cx<0)
        cx = 0;
      if(cy>239)
        cy = 239;
      if(cx>319)
        cx = 319;
      
      gp2x_video_cursor_move(cx,cy);
      gp2x_video_cursor_setalpha(0xF,0xA);

      gp2x_timer_delay(1);
    }
    
    last_pad = pad;
    

    
  }
}

void gp2x_sound_frame(void *blah, void *buff, int samples) {}

